'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var Vue = _interopDefault(require('vue'));
var VueRouter = _interopDefault(require('vue-router'));
var Vuex = _interopDefault(require('vuex'));
var assign = _interopDefault(require('object-assign'));
var vuexRouterSync = require('vuex-router-sync');

var Store = Vuex.Store;
var mapActions = Vuex.mapActions;
var mapGetters = Vuex.mapGetters;
var mapMutations = Vuex.mapMutations;
var mapState = Vuex.mapState;

Vue.use(VueRouter);
Vue.use(Vuex);

var DEV = process.env.NODE_ENV === 'development';

var EVA = function EVA(options) {
  if ( options === void 0 ) options = {};

  this.routes = [];
  this.options = options;
};
EVA.prototype.use = function use () {
    var args = [], len = arguments.length;
    while ( len-- ) args[ len ] = arguments[ len ];

  Vue.use.apply(Vue, args);
};
EVA.prototype.model = function model (m) {
  var name = m && m.name;
  // initial store instance
  if (!this.$store) {
    if (name) {
      // to initialize an empty store
      // will be used to register namespaced models
      this.$store = new Store();
    } else {
      // to initialize a store with a top-level model
      // early return since we don't need to register namespaced model
      this.$store = new Store(m);
      return
    }
  }
  if (DEV && !name) {
    throw new Error('[eva] Only one top-level model is allowed!')
  }
  // once the store intance is initialized
  // add namespaced model here
  this.$store.registerModule(name, m);
};
EVA.prototype.route = function route (path, component, children) {
  return {
    path: path,
    component: component,
    children: children
  }
};
EVA.prototype.router = function router (handleRoute) {
  this.routes = handleRoute(this.route);
  this.$router = new VueRouter({
    routes: this.routes,
    mode: this.options.mode
  });
};
EVA.prototype.start = function start (app, mountTo) {
  if (this.$store && this.$router) {
    vuexRouterSync.sync(this.$store, this.$router);
  }
  this.vm = new Vue(assign({
    store: this.$store,
    router: this.$router
  }, app));
  this.vm.$mount(mountTo);
};

exports['default'] = EVA;
exports.mapActions = mapActions;
exports.mapGetters = mapGetters;
exports.mapMutations = mapMutations;
exports.mapState = mapState;
